/* USER CODE BEGIN Header */
/**
 ******************************************************************************
 * @file           : main.c
 * @brief          : Main program body
 ******************************************************************************
 * @attention
 *
 * Copyright (c) 2023 STMicroelectronics.
 * All rights reserved.
 *
 * This software is licensed under terms that can be found in the LICENSE file
 * in the root directory of this software component.
 * If no LICENSE file comes with this software, it is provided AS-IS.
 *
 ******************************************************************************
 */
/* USER CODE END Header */
/* Includes ------------------------------------------------------------------*/
#include "main.h"
#include "st7735.h"
#include "fonts.h"
#include "stdio.h"
#include "stdbool.h"
#include "string.h"
#include "stdlib.h"

/* Private includes ----------------------------------------------------------*/
/* USER CODE BEGIN Includes */

/* USER CODE END Includes */

/* Private typedef -----------------------------------------------------------*/
/* USER CODE BEGIN PTD */

/* USER CODE END PTD */

/* Private define ------------------------------------------------------------*/
/* USER CODE BEGIN PD */
// All commands
#define NOP     	0x00 // DC = 0
#define SWRESET 	0x01 // DC = 0
#define RDDID   	0x04 // DC = 0 + 1
#define RDDST   	0x09 // DC = 0 + 1
#define RDDPM   	0x0A // DC = 0 + 1
#define RDDMADCTL   0x0B // DC = 0 + 1
#define RDDCOLMOD   0x0C // DC = 0 + 1
#define RDDIM   	0x0D // DC = 0 + 1
#define RDDSM  		0x0E // DC = 0 + 1
#define SLPIN   	0x10 // DC = 0
#define SLPOUT  	0x11 // DC = 0
#define PTLON   	0x12 // DC = 0
#define NORON   	0x13 // DC = 0
#define INVOFF   	0x20 // DC = 0
#define INVON   	0x21 // DC = 0
#define GAMSET 		0x26 // DC = 0 + 1
#define DISPOFF 	0x28 // DC = 0
#define DISPON  	0x29 // DC = 0
#define CASET   	0x2A // DC = 0 + 1
#define RASET   	0x2B // DC = 0 + 1
#define RAMWR   	0x2C // DC = 0 + 1
#define RAMRD   	0x2E // DC = 0 + 1
#define PTLAR   	0x30 // DC = 0 + 1
#define TEOFF  		0x34 // DC = 0
#define TEON  		0x35 // DC = 0 + 1
#define MADCTL  	0x36 // DC = 0 + 1
#define IDMOFF  	0x38 // DC = 0
#define IDMON  		0x39 // DC = 0
#define COLMOD  	0x3A // DC = 0 + 1
#define FRMCTR1  	0xB1 // DC = 0 + 1
#define FRMCTR2  	0xB2 // DC = 0 + 1
#define FRMCTR3  	0xB3 // DC = 0 + 1
#define INVCTR  	0xB4 // DC = 0 + 1
#define DISSET5  	0xB6 // DC = 0 + 1
#define PWCTR1  	0xC0 // DC = 0 + 1
#define PWCTR2  	0xC1 // DC = 0 + 1
#define PWCTR3  	0xC2 // DC = 0 + 1
#define PWCTR4  	0xC3 // DC = 0 + 1
#define PWCTR5  	0xC4 // DC = 0 + 1
#define VMCTR1  	0xC5 // DC = 0 + 1
#define VMOFCTR  	0xC7 // DC = 0 + 1
#define WRID2	  	0xD1 // DC = 0 + 1
#define WRID3   	0xD2 // DC = 0 + 1
#define PWCTR6   	0xFC // DC = 0 + 1
#define NVCTR1   	0xD9 // DC = 0 + 1
#define NVCTR2   	0xDE // DC = 0 + 1
#define NVCTR3   	0xDF // DC = 0 + 1
#define RDID1   	0xDA // DC = 0 + 1
#define RDID2   	0xDB // DC = 0 + 1
#define RDID3   	0xDC // DC = 0 + 1
#define GAMCTRP1   	0xE0 // DC = 0 + 1
#define GAMCTRN1   	0xE1 // DC = 0 + 1
#define EXTCTRL   	0xF0 // DC = 0 + 1
#define VCOM4L   	0xFF // DC = 0 + 1
/* USER CODE END PD */

/* Private macro -------------------------------------------------------------*/
/* USER CODE BEGIN PM */

/* USER CODE END PM */

/* Private variables ---------------------------------------------------------*/
SPI_HandleTypeDef hspi2;

TIM_HandleTypeDef htim6;
TIM_HandleTypeDef htim14;

UART_HandleTypeDef huart2;

/* USER CODE BEGIN PV */

/* USER CODE END PV */

/* Private function prototypes -----------------------------------------------*/
void SystemClock_Config(void);
static void MX_GPIO_Init(void);
static void MX_USART2_UART_Init(void);
static void MX_TIM14_Init(void);
static void MX_SPI2_Init(void);
static void MX_TIM6_Init(void);

/* USER CODE BEGIN PFP */
void TFT_start_init(void);
void SPI_sendCommand(uint8_t command);
void SPI_sendData(uint8_t *buff, size_t buff_size);
void Set_DS18B20_Pin_as_Output();
void Set_DS18B20_Pin_as_Input();
void DS18B20_init();
void DS18B20_Read_ROM();
void Write_Byte(uint8_t byte_to_send_to_DS18B20);
uint8_t Read_Byte(void);
uint8_t CRC_Calculator(uint8_t *mas, uint8_t Len);
void DS18B20_Read_Scratchpad();
void DS18B20_Decode_Scratchpad();
void DS18B20_Write_Scratchpad();
void DS18B20_Decode_Temperature();
/* USER CODE END PFP */

/* Private user code ---------------------------------------------------------*/
/* USER CODE BEGIN 0 */
static unsigned short DS18B20_Sensor_present = 0; 		// Init global variable
uint8_t ROM_address[] = { 0, 0, 0, 0, 0, 0, 0, 0 };		// In total 8 bytes
uint8_t Scratchpad[] = { 0, 0, 0, 0, 0, 0, 0, 0, 0 };	// In total 9 bytes
uint8_t crc;
bool ROM_OK_Flag;
uint8_t TH_reg;
uint8_t TL_reg;
uint8_t Cfg_Reg;
/* USER CODE END 0 */

/**
 * @brief  The application entry point.
 * @retval int
 */
int main(void) {
	/* USER CODE BEGIN 1 */

	/* USER CODE END 1 */

	/* MCU Configuration--------------------------------------------------------*/

	/* Reset of all peripherals, Initializes the Flash interface and the Systick. */
	HAL_Init();

	/* USER CODE BEGIN Init */

	/* USER CODE END Init */

	/* Configure the system clock */
	SystemClock_Config();

	/* USER CODE BEGIN SysInit */

	/* USER CODE END SysInit */

	/* Initialize all configured peripherals */
	MX_GPIO_Init();
	MX_USART2_UART_Init();
	MX_TIM14_Init();
	MX_SPI2_Init();
	MX_TIM6_Init();
	/* USER CODE BEGIN 2 */
	TFT_start_init();
	HAL_TIM_Base_Start(&htim14); 				// TFT BACKLIGHT timer start, clock - APB1
	HAL_TIM_PWM_Start(&htim14, TIM_CHANNEL_1);	// TFT BACKLIGHT start PWM
	HAL_TIM_Base_Start(&htim6);					// Timer for us delay start

	/* USER CODE END 2 */

	/* Infinite loop */
	/* USER CODE BEGIN WHILE */
	while (1) {
		/* USER CODE END WHILE */

		/*
		 * All the commands:
		 *
		 * ROM commands
		 * 33h - Read ROM : done
		 * 55h - Match ROM : done
		 * F0h - Search ROM :
		 * ECh - Alarm search :
		 * CCh - Skip ROM : tried, not used
		 *
		 * Function commands
		 * 44h - Convert temperatures : done
		 * BEh - Read scratchpad : done
		 * 4Eh - Write scratchpad :
		 * 48h - Copy scratchpad :
		 * B8h - Recall Th, Tl and configuration register from EEPROM to scratchpad :
		 * B4h - Read power supply :
		 *
		 *  Parasite power is: OFF
		 *	Device 0 Address: 28 3B 1B 1E 0D 00 00 3A
		 *	Device 0 Resolution: 9
		 *	Temp C: 25.50 Temp F: 77.90
		 *
		 *
		 *
		 */

		DS18B20_init();																					// Init procedure

		if (DS18B20_Sensor_present == 1) {
			ST7735_WriteString(35, 0, "PONG", Font_7x10, ST7735_YELLOW, ST7735_BLUE);
			DS18B20_Read_ROM();																			// Read ROM procedure

			if (ROM_address[0] == 0x10) {
				ST7735_WriteString(0, 12, "DS18S20 found", Font_7x10, ST7735_YELLOW, ST7735_BLUE);
			} else {

				if (ROM_address[0] == 0x22) {
					ST7735_WriteString(0, 12, "DS1822 found", Font_7x10, ST7735_YELLOW, ST7735_BLUE);
				} else {
					if (ROM_address[0] == 0x28) {
						ST7735_WriteString(0, 12, "DS18B20 found", Font_7x10, ST7735_YELLOW, ST7735_BLUE);
					}
				}
			}

			DS18B20_Read_Scratchpad();																	// Read scratchpad
			DS18B20_Write_Scratchpad();																	// Write scratchpad procedure
			DS18B20_Decode_Scratchpad();																// Decode scratchpad to LCD procedure
			DS18B20_Decode_Temperature();																	// Read temperature and show on LCD

		} else {
			ST7735_WriteString(35, 0, "TIME OUT", Font_7x10, ST7735_YELLOW, ST7735_BLUE);
			HAL_GPIO_WritePin(GPIOA, HEARTBEAT_Pin, 0);  // Yellow LED

		}
		HAL_Delay(800);

		/* USER CODE BEGIN 3 */
	}
	/* USER CODE END 3 */
}

/**
 * @brief System Clock Configuration
 * @retval None
 */
void SystemClock_Config(void) {
	RCC_OscInitTypeDef RCC_OscInitStruct = { 0 };
	RCC_ClkInitTypeDef RCC_ClkInitStruct = { 0 };

	/** Configure the main internal regulator output voltage
	 */
	__HAL_RCC_PWR_CLK_ENABLE();
	__HAL_PWR_VOLTAGESCALING_CONFIG(PWR_REGULATOR_VOLTAGE_SCALE1);

	/** Initializes the RCC Oscillators according to the specified parameters
	 * in the RCC_OscInitTypeDef structure.
	 */
	RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_HSI;
	RCC_OscInitStruct.HSIState = RCC_HSI_ON;
	RCC_OscInitStruct.HSICalibrationValue = RCC_HSICALIBRATION_DEFAULT;
	RCC_OscInitStruct.PLL.PLLState = RCC_PLL_ON;
	RCC_OscInitStruct.PLL.PLLSource = RCC_PLLSOURCE_HSI;
	RCC_OscInitStruct.PLL.PLLM = 8;
	RCC_OscInitStruct.PLL.PLLN = 180;
	RCC_OscInitStruct.PLL.PLLP = RCC_PLLP_DIV2;
	RCC_OscInitStruct.PLL.PLLQ = 2;
	RCC_OscInitStruct.PLL.PLLR = 2;
	if (HAL_RCC_OscConfig(&RCC_OscInitStruct) != HAL_OK) {
		Error_Handler();
	}

	/** Activate the Over-Drive mode
	 */
	if (HAL_PWREx_EnableOverDrive() != HAL_OK) {
		Error_Handler();
	}

	/** Initializes the CPU, AHB and APB buses clocks
	 */
	RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_HCLK | RCC_CLOCKTYPE_SYSCLK | RCC_CLOCKTYPE_PCLK1 | RCC_CLOCKTYPE_PCLK2;
	RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_PLLCLK;
	RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
	RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV4;
	RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV2;

	if (HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_5) != HAL_OK) {
		Error_Handler();
	}
}

/**
 * @brief SPI2 Initialization Function
 * @param None
 * @retval None
 */
static void MX_SPI2_Init(void) {

	/* USER CODE BEGIN SPI2_Init 0 */

	/* USER CODE END SPI2_Init 0 */

	/* USER CODE BEGIN SPI2_Init 1 */

	/* USER CODE END SPI2_Init 1 */
	/* SPI2 parameter configuration*/
	hspi2.Instance = SPI2;
	hspi2.Init.Mode = SPI_MODE_MASTER;
	hspi2.Init.Direction = SPI_DIRECTION_2LINES;
	hspi2.Init.DataSize = SPI_DATASIZE_8BIT;
	hspi2.Init.CLKPolarity = SPI_POLARITY_LOW;
	hspi2.Init.CLKPhase = SPI_PHASE_1EDGE;
	hspi2.Init.NSS = SPI_NSS_SOFT;
	hspi2.Init.BaudRatePrescaler = SPI_BAUDRATEPRESCALER_2;
	hspi2.Init.FirstBit = SPI_FIRSTBIT_MSB;
	hspi2.Init.TIMode = SPI_TIMODE_DISABLE;
	hspi2.Init.CRCCalculation = SPI_CRCCALCULATION_DISABLE;
	hspi2.Init.CRCPolynomial = 10;
	if (HAL_SPI_Init(&hspi2) != HAL_OK) {
		Error_Handler();
	}
	/* USER CODE BEGIN SPI2_Init 2 */

	/* USER CODE END SPI2_Init 2 */

}

/**
 * @brief TIM6 Initialization Function
 * @param None
 * @retval None
 */
static void MX_TIM6_Init(void) {

	/* USER CODE BEGIN TIM6_Init 0 */

	/* USER CODE END TIM6_Init 0 */

	TIM_MasterConfigTypeDef sMasterConfig = { 0 };

	/* USER CODE BEGIN TIM6_Init 1 */

	/* USER CODE END TIM6_Init 1 */
	htim6.Instance = TIM6;
	htim6.Init.Prescaler = 90 - 1;
	htim6.Init.CounterMode = TIM_COUNTERMODE_UP;
	htim6.Init.Period = 65535;
	htim6.Init.AutoReloadPreload = TIM_AUTORELOAD_PRELOAD_DISABLE;
	if (HAL_TIM_Base_Init(&htim6) != HAL_OK) {
		Error_Handler();
	}
	sMasterConfig.MasterOutputTrigger = TIM_TRGO_RESET;
	sMasterConfig.MasterSlaveMode = TIM_MASTERSLAVEMODE_DISABLE;
	if (HAL_TIMEx_MasterConfigSynchronization(&htim6, &sMasterConfig) != HAL_OK) {
		Error_Handler();
	}
	/* USER CODE BEGIN TIM6_Init 2 */

	/* USER CODE END TIM6_Init 2 */

}

/**
 * @brief TIM14 Initialization Function
 * @param None
 * @retval None
 */
static void MX_TIM14_Init(void) {

	/* USER CODE BEGIN TIM14_Init 0 */

	/* USER CODE END TIM14_Init 0 */

	TIM_OC_InitTypeDef sConfigOC = { 0 };

	/* USER CODE BEGIN TIM14_Init 1 */

	/* USER CODE END TIM14_Init 1 */
	htim14.Instance = TIM14;
	htim14.Init.Prescaler = 900 - 1;
	htim14.Init.CounterMode = TIM_COUNTERMODE_UP;
	htim14.Init.Period = 100 - 1;
	htim14.Init.ClockDivision = TIM_CLOCKDIVISION_DIV1;
	htim14.Init.AutoReloadPreload = TIM_AUTORELOAD_PRELOAD_ENABLE;
	if (HAL_TIM_Base_Init(&htim14) != HAL_OK) {
		Error_Handler();
	}
	if (HAL_TIM_PWM_Init(&htim14) != HAL_OK) {
		Error_Handler();
	}
	sConfigOC.OCMode = TIM_OCMODE_PWM1;
	sConfigOC.Pulse = 100 - 1;
	sConfigOC.OCPolarity = TIM_OCPOLARITY_LOW;
	sConfigOC.OCFastMode = TIM_OCFAST_DISABLE;
	if (HAL_TIM_PWM_ConfigChannel(&htim14, &sConfigOC, TIM_CHANNEL_1) != HAL_OK) {
		Error_Handler();
	}
	__HAL_TIM_DISABLE_OCxPRELOAD(&htim14, TIM_CHANNEL_1);
	/* USER CODE BEGIN TIM14_Init 2 */

	/* USER CODE END TIM14_Init 2 */
	HAL_TIM_MspPostInit(&htim14);

}

/**
 * @brief USART2 Initialization Function
 * @param None
 * @retval None
 */
static void MX_USART2_UART_Init(void) {

	/* USER CODE BEGIN USART2_Init 0 */

	/* USER CODE END USART2_Init 0 */

	/* USER CODE BEGIN USART2_Init 1 */

	/* USER CODE END USART2_Init 1 */
	huart2.Instance = USART2;
	huart2.Init.BaudRate = 115200;
	huart2.Init.WordLength = UART_WORDLENGTH_8B;
	huart2.Init.StopBits = UART_STOPBITS_1;
	huart2.Init.Parity = UART_PARITY_NONE;
	huart2.Init.Mode = UART_MODE_TX_RX;
	huart2.Init.HwFlowCtl = UART_HWCONTROL_NONE;
	huart2.Init.OverSampling = UART_OVERSAMPLING_16;
	if (HAL_UART_Init(&huart2) != HAL_OK) {
		Error_Handler();
	}
	/* USER CODE BEGIN USART2_Init 2 */

	/* USER CODE END USART2_Init 2 */

}

/**
 * @brief GPIO Initialization Function
 * @param None
 * @retval None
 */
static void MX_GPIO_Init(void) {
	GPIO_InitTypeDef GPIO_InitStruct = { 0 };
	/* USER CODE BEGIN MX_GPIO_Init_1 */
	/* USER CODE END MX_GPIO_Init_1 */

	/* GPIO Ports Clock Enable */
	__HAL_RCC_GPIOC_CLK_ENABLE();
	__HAL_RCC_GPIOH_CLK_ENABLE();
	__HAL_RCC_GPIOA_CLK_ENABLE();
	__HAL_RCC_GPIOB_CLK_ENABLE();

	/*Configure GPIO pin Output Level */
	HAL_GPIO_WritePin(GPIOA,
	DS18B20_Data_Pin | HEARTBEAT_Pin | SD_CS_Pin | TFT_CS_Pin, GPIO_PIN_RESET);

	/*Configure GPIO pin Output Level */
	HAL_GPIO_WritePin(TFT_D_C_GPIO_Port, TFT_D_C_Pin, GPIO_PIN_RESET);

	/*Configure GPIO pin Output Level */
	HAL_GPIO_WritePin(TFT_Reset_GPIO_Port, TFT_Reset_Pin, GPIO_PIN_RESET);

	/*Configure GPIO pin : B1_Pin */
	GPIO_InitStruct.Pin = B1_Pin;
	GPIO_InitStruct.Mode = GPIO_MODE_IT_FALLING;
	GPIO_InitStruct.Pull = GPIO_NOPULL;
	HAL_GPIO_Init(B1_GPIO_Port, &GPIO_InitStruct);

	/*Configure GPIO pins : DS18B20_Data_Pin HEARTBEAT_Pin SD_CS_Pin TFT_CS_Pin */
	GPIO_InitStruct.Pin = DS18B20_Data_Pin | HEARTBEAT_Pin | SD_CS_Pin | TFT_CS_Pin;
	GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
	GPIO_InitStruct.Pull = GPIO_NOPULL;
	GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
	HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);

	/*Configure GPIO pin : TFT_D_C_Pin */
	GPIO_InitStruct.Pin = TFT_D_C_Pin;
	GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
	GPIO_InitStruct.Pull = GPIO_NOPULL;
	GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
	HAL_GPIO_Init(TFT_D_C_GPIO_Port, &GPIO_InitStruct);

	/*Configure GPIO pin : TFT_Reset_Pin */
	GPIO_InitStruct.Pin = TFT_Reset_Pin;
	GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
	GPIO_InitStruct.Pull = GPIO_NOPULL;
	GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
	HAL_GPIO_Init(TFT_Reset_GPIO_Port, &GPIO_InitStruct);

	/* USER CODE BEGIN MX_GPIO_Init_2 */
	/* USER CODE END MX_GPIO_Init_2 */
}

/* USER CODE BEGIN 4 */
//============================================================================
void TFT_start_init(void) {

	HAL_GPIO_WritePin(GPIOB, TFT_Reset_Pin, RESET);		// RS - active low, thus resetting display with low
	HAL_Delay(20);										// Min reset time 10 ms, doubled to be sure
	HAL_GPIO_WritePin(GPIOB, TFT_Reset_Pin, SET); 		// RS - active low, thus defaulting to high
	HAL_Delay(120);								   		// After HW reset time 120 ms

	SPI_sendCommand(SWRESET);							// 0x01
	HAL_Delay(120);										// After SWRESET time 120 ms

	SPI_sendCommand(SLPOUT);							// 0x11
	HAL_Delay(120);										// After SLPOUT time 120 ms

	SPI_sendCommand(FRMCTR1);							// 0xB1
	uint8_t FRMCTR1data[] = { 0x03, 0x01, 0x01 };
	SPI_sendData(FRMCTR1data, sizeof(FRMCTR1data));

	SPI_sendCommand(FRMCTR2);							// 0xB2
	uint8_t FRMCTR2data[] = { 0x03, 0x01, 0x01 };
	SPI_sendData(FRMCTR2data, sizeof(FRMCTR2data));

	SPI_sendCommand(FRMCTR3);							// 0xB3
	uint8_t FRMCTR3data[] = { 0x03, 0x01, 0x01, 0x03, 0x01, 0x01 };
	SPI_sendData(FRMCTR3data, sizeof(FRMCTR3data));

	SPI_sendCommand(INVCTR);							// 0xB4
	uint8_t INVCTRdata[] = { 0x07 };
	SPI_sendData(INVCTRdata, sizeof(INVCTRdata));

	SPI_sendCommand(PWCTR1);							// 0xC0
	uint8_t PWCTR1data[] = { 0x1D, 0x70, };				// 1D = 3,35 V 0x11 = 1 uA
	SPI_sendData(PWCTR1data, sizeof(PWCTR1data));

	SPI_sendCommand(PWCTR2);							// 0xC1
	uint8_t PWCTR2data[] = { 0x05 };					//
	SPI_sendData(PWCTR2data, sizeof(PWCTR2data));

	SPI_sendCommand(PWCTR3);							// 0xC2
	uint8_t PWCTR3data[] = { 0x03, 0x00 };				//
	SPI_sendData(PWCTR3data, sizeof(PWCTR3data));

	SPI_sendCommand(PWCTR4);							// 0xC3
	uint8_t PWCTR4data[] = { 0x03, 0x00 };				//
	SPI_sendData(PWCTR4data, sizeof(PWCTR4data));

	SPI_sendCommand(PWCTR5);							// 0xC4
	uint8_t PWCTR5data[] = { 0x03, 0x00 };				//
	SPI_sendData(PWCTR5data, sizeof(PWCTR5data));

	SPI_sendCommand(VMCTR1);							// 0xC4
	uint8_t VMCTR1data[] = { 0x20, 0x64 };				//
	SPI_sendData(VMCTR1data, sizeof(VMCTR1data));

	SPI_sendCommand(INVOFF);							// Color inversion

	SPI_sendCommand(MADCTL);							// 0x36
	uint8_t MADCTLdata[] = { 0x08 };					//
	SPI_sendData(MADCTLdata, sizeof(MADCTLdata));

	SPI_sendCommand(COLMOD);							// 0x3A
	uint8_t COLMODdata[] = { 0x05 };					//
	SPI_sendData(COLMODdata, sizeof(COLMODdata));

	SPI_sendCommand(CASET);								// 0x2A
	uint8_t CASETdata[] = { 0x00, 0x00, 0x00, 0x00 };	//
	SPI_sendData(CASETdata, sizeof(CASETdata));

	SPI_sendCommand(RASET);								// 0x2B
	uint8_t RASETdata[] = { 0x00, 0x00, 0x00, 0x00 };	//
	SPI_sendData(RASETdata, sizeof(RASETdata));

	SPI_sendCommand(NORON);								// 0x13
	HAL_Delay(10);

	SPI_sendCommand(DISPON);							// 0x29
	ST7735_FillScreen(ST7735_BLUE);  					// Set blue background
}

void SPI_sendCommand(uint8_t command) {
	HAL_GPIO_WritePin(GPIOA, TFT_CS_Pin, RESET); 						// Chipselect, active low - selected
	HAL_GPIO_WritePin(GPIOC, TFT_D_C_Pin, RESET);						// Command register - low
	HAL_SPI_Transmit(&hspi2, &command, sizeof(command), HAL_MAX_DELAY);	// Send command
	HAL_GPIO_WritePin(GPIOA, TFT_CS_Pin, SET);							// Chipselect, active low - unselected

}
//----------------------------------------------------------------------------
void SPI_sendData(uint8_t *buff, size_t buff_size) {
	HAL_GPIO_WritePin(GPIOA, TFT_CS_Pin, RESET); 						// Chipselect, active low - selected
	HAL_GPIO_WritePin(GPIOC, TFT_D_C_Pin, SET);							// Data register - high
	HAL_SPI_Transmit(&hspi2, buff, buff_size, HAL_MAX_DELAY);			// Send data
	HAL_GPIO_WritePin(GPIOA, TFT_CS_Pin, SET);							// Chipselect, active low - unselected
}

void Delay_in_us(uint16_t us) {											// delay function variable us, 16 bit length max
	__HAL_TIM_SET_COUNTER(&htim6, 0);  									// set the counter value a 0
	while ((uint16_t) __HAL_TIM_GET_COUNTER(&htim6) < us)
		;																// wait for the counter to count to us set in the variable
}

void DS18B20_init() {
	ST7735_WriteString(0, 0, "PING:          ", Font_7x10, ST7735_YELLOW, ST7735_BLUE);
	Set_DS18B20_Pin_as_Output();
	HAL_GPIO_WritePin(GPIOA, DS18B20_Data_Pin, 0); 						// And as specified in the sensor datasheet pulling the line low for
	Delay_in_us(480);													// 480 us and then
	Set_DS18B20_Pin_as_Input();											// make it input

	Delay_in_us(70);													// Datasheet says sensor will respond after 15-60 us, so 70 is a reasonable value

	if (HAL_GPIO_ReadPin(GPIOA, DS18B20_Data_Pin) == 0) {				// After waiting the 70 us checking if we see the low level on the line.
		Delay_in_us(410);								// From datasheet, Master RX = 480 us, counting since pulling the pin high, so delay 70us+410us = 480us
		DS18B20_Sensor_present = 1;										// Light up the LED if sensor detected
	}

	else {

		Delay_in_us(410);								// From datasheet, Master RX = 480 us, counting since pulling the pin high, so delay 70us+410us = 480us
		DS18B20_Sensor_present = 0;										// Nothing on the LED if sensor not detected
																		// DP wee need here some error handler ? Maybe a red led or something :) ?
	}
	Set_DS18B20_Pin_as_Input();
}

void DS18B20_Read_ROM() {

	if (DS18B20_Sensor_present == 1) {

		Write_Byte(0x33);												// 0x33 = 0b 0011 0011 ; 0xF0 = 0b 1111 0000

		for (int i = 0; i < 8; i++) {
			ROM_address[i] = Read_Byte();
		}

		ST7735_WriteString(0, 24, "ROM Address", Font_7x10, ST7735_YELLOW, ST7735_BLUE);

		const char *ROM_Address = (char*) ROM_address;

		char ROM_Address_to_print[sizeof(ROM_address) * 2 + 1];

		snprintf(ROM_Address_to_print, sizeof(ROM_Address_to_print), "%02X %02X %02X %02X", ROM_Address[0], ROM_Address[1], ROM_Address[2], ROM_Address[3]);
		ST7735_WriteString(0, 36, ROM_Address_to_print, Font_7x10, ST7735_YELLOW, ST7735_BLUE);
		snprintf(ROM_Address_to_print, sizeof(ROM_Address_to_print), "%02X %02X %02X", ROM_Address[4], ROM_Address[5], ROM_Address[6]);
		ST7735_WriteString(0, 48, ROM_Address_to_print, Font_7x10, ST7735_YELLOW, ST7735_BLUE);

		ST7735_FillRectangle(81, 24, 45, 32, ST7735_BLACK); 			// X, Y, Width, Height
		ST7735_WriteString(82, 25, "ROMCRC", Font_7x10, ST7735_WHITE, ST7735_BLACK);

		// CRC check of ROM:
		CRC_Calculator(ROM_address, 7);

		if (crc == ROM_address[7]) {																	// If calculated and received CRC are equal

			snprintf(ROM_Address_to_print, sizeof(ROM_Address_to_print), "%02X", ROM_Address[7]);		// Convert data to printable
			ST7735_WriteString(63, 48, ROM_Address_to_print, Font_7x10, ST7735_GREEN, ST7735_BLUE);		// and print CRC byte in green
			ST7735_WriteString(92, 37, "OK", Font_11x18, ST7735_WHITE, ST7735_BLACK);					// and write "OK"
			ROM_OK_Flag = 1;
		}

		else {																							// If calculated and received CRC are not equal

			snprintf(ROM_Address_to_print, sizeof(ROM_Address_to_print), "%02X", ROM_Address[7]);		// convert data to printable
			ST7735_WriteString(63, 48, ROM_Address_to_print, Font_7x10, ST7735_RED, ST7735_BLUE);		// and print CRC byte in red

			char CRC_to_print[sizeof(crc) * 2 + 1];														// Convert calculated
			snprintf(CRC_to_print, sizeof(CRC_to_print), "%02X", crc);									// CRC byte to printable
			ST7735_WriteString(92, 37, CRC_to_print, Font_11x18, ST7735_RED, ST7735_BLACK);				// and write it in red
			ROM_OK_Flag = 0;
		}

	}

	else {
		ST7735_WriteString(35, 0, "ERROR1", Font_7x10, ST7735_YELLOW, ST7735_BLUE);
	}

}

void DS18B20_Read_Scratchpad() {

	DS18B20_init();

	if (DS18B20_Sensor_present == 1) {
		if (ROM_OK_Flag == 1) {
			ST7735_WriteString(35, 0, "PONG", Font_7x10, ST7735_YELLOW, ST7735_BLUE);

			Write_Byte(0x55);																			// Match ROM command [0x55]
			for (int i = 0; i < 8; i++) {																// Send previously detected
				Write_Byte(ROM_address[i]);																	// ROM, all 8 bytes
			}
			Write_Byte(0x44);																			//Convert Temperature [44h] to update the scratchpad

			DS18B20_init();

			if (DS18B20_Sensor_present == 1) {
				ST7735_WriteString(35, 0, "PONG", Font_7x10, ST7735_YELLOW, ST7735_BLUE);

				Write_Byte(0x55);																		// Match ROM command [0x55]
				for (int i = 0; i < 8; i++) {															// Send previously detected
					Write_Byte(ROM_address[i]);																// ROM, all 8 bytes
				}
				Write_Byte(0xBE);																		// Read scratchpad command

				for (uint8_t i = 0; i < 9; i++) {
					Scratchpad[i] = Read_Byte();
				}

				ST7735_WriteString(0, 60, "Scratchpad:", Font_7x10, ST7735_YELLOW, ST7735_BLUE);

				TH_reg = Scratchpad[2];
				TL_reg = Scratchpad[3];
				Cfg_Reg = Scratchpad[4];

				const char *ScratchPad = (char*) Scratchpad;

				char ScratchPad_to_print[sizeof(Scratchpad) * 2 + 1];

				snprintf(ScratchPad_to_print, sizeof(ScratchPad_to_print), "%02X %02X", ScratchPad[0], ScratchPad[1]);
				ST7735_WriteString(0, 72, ScratchPad_to_print, Font_7x10, ST7735_RED, ST7735_BLUE);

				snprintf(ScratchPad_to_print, sizeof(ScratchPad_to_print), "%02X %02X", ScratchPad[2], ScratchPad[3]);
				ST7735_WriteString(41, 72, ScratchPad_to_print, Font_7x10, ST7735_YELLOW, ST7735_BLUE);

				snprintf(ScratchPad_to_print, sizeof(ScratchPad_to_print), "%02X", ScratchPad[4]);
				ST7735_WriteString(0, 84, ScratchPad_to_print, Font_7x10, ST7735_YELLOW, ST7735_BLUE);

				snprintf(ScratchPad_to_print, sizeof(ScratchPad_to_print), "%02X %02X %02X", ScratchPad[5], ScratchPad[6], ScratchPad[7]);
				ST7735_WriteString(21, 84, ScratchPad_to_print, Font_7x10, ST7735_BLACK, ST7735_BLUE);

				ST7735_FillRectangle(81, 61, 45, 42, ST7735_BLACK); 												// X, Y, Width, Height
				ST7735_WriteString(82, 62, "SCRPAD", Font_7x10, ST7735_WHITE, ST7735_BLACK);
				ST7735_WriteString(92, 74, "CRC", Font_7x10, ST7735_WHITE, ST7735_BLACK);

				CRC_Calculator(Scratchpad, 8);

				if (crc == Scratchpad[8]) {																		// If calculated and received CRC are equal

					snprintf(ScratchPad_to_print, sizeof(ScratchPad_to_print), "%02X", Scratchpad[8]);			// Convert data to printable
					ST7735_WriteString(0, 96, ScratchPad_to_print, Font_7x10, ST7735_GREEN, ST7735_BLUE);		// and print CRC byte in green
					ST7735_WriteString(92, 84, "OK", Font_11x18, ST7735_WHITE, ST7735_BLACK);					// and write "OK"

				} else {																						// If calculated and received CRC are not equal

					snprintf(ScratchPad_to_print, sizeof(ScratchPad_to_print), "%02X", Scratchpad[8]);			// convert data to printable
					ST7735_WriteString(0, 96, ScratchPad_to_print, Font_7x10, ST7735_RED, ST7735_BLUE);			// and print CRC byte in red

					char CRC_to_print[sizeof(crc) * 2 + 1];														// Convert calculated
					snprintf(CRC_to_print, sizeof(CRC_to_print), "%02X", crc);									// CRC byte to printable
					ST7735_WriteString(92, 84, CRC_to_print, Font_11x18, ST7735_RED, ST7735_BLACK);				// and write it in red
				}

			} else {
				ST7735_WriteString(35, 0, "ERROR4", Font_7x10, ST7735_YELLOW, ST7735_BLUE);
			}

		} else {
			ST7735_WriteString(35, 0, "ERROR3", Font_7x10, ST7735_YELLOW, ST7735_BLUE);
		}

	} else {
		ST7735_WriteString(35, 0, "ERROR2", Font_7x10, ST7735_YELLOW, ST7735_BLUE);
	}
}

void DS18B20_Decode_Scratchpad() {

	if (DS18B20_Sensor_present == 1) {
		if (ROM_OK_Flag == 1) {

			char Config_Register_to_print[sizeof(Cfg_Reg) * 2 + 1];

			snprintf(Config_Register_to_print, sizeof(Config_Register_to_print), "%02X", Cfg_Reg);

			char hex2bin[8] = "";

			for (int i = 0; i < 2; i++) {

				switch (Config_Register_to_print[i]) {
				case '0':
					strcat(hex2bin, "0000");
					break;
				case '1':
					strcat(hex2bin, "0001");
					break;
				case '2':
					strcat(hex2bin, "0010");
					break;
				case '3':
					strcat(hex2bin, "0011");
					break;
				case '4':
					strcat(hex2bin, "0100");
					break;
				case '5':
					strcat(hex2bin, "0101");
					break;
				case '6':
					strcat(hex2bin, "0110");
					break;
				case '7':
					strcat(hex2bin, "0111");
					break;
				case '8':
					strcat(hex2bin, "1000");
					break;
				case '9':
					strcat(hex2bin, "1001");
					break;
				case 'a':
				case 'A':
					strcat(hex2bin, "1010");
					break;
				case 'b':
				case 'B':
					strcat(hex2bin, "1011");
					break;
				case 'c':
				case 'C':
					strcat(hex2bin, "1100");
					break;
				case 'd':
				case 'D':
					strcat(hex2bin, "1101");
					break;
				case 'e':
				case 'E':
					strcat(hex2bin, "1110");
					break;
				case 'f':
				case 'F':
					strcat(hex2bin, "1111");
					break;
				default:
					ST7735_WriteString(35, 0, "ERROR5", Font_7x10, ST7735_YELLOW, ST7735_BLUE);
				}
			}

			char TH_reg_to_print[sizeof(TH_reg) * 3 + 1];
			snprintf(TH_reg_to_print, sizeof(TH_reg_to_print), "%3u", TH_reg);

			char TL_reg_to_print[sizeof(TL_reg) * 3 + 1];
			snprintf(TL_reg_to_print, sizeof(TL_reg_to_print), "%3u", TL_reg);

			char TL_TH_to_LCD[18] = ""; // TH: 123C, TL: 456C

			strcat(TL_TH_to_LCD, "TH:");
			strcat(TL_TH_to_LCD, TH_reg_to_print);
			strcat(TL_TH_to_LCD, "C, TL:");
			strcat(TL_TH_to_LCD, TL_reg_to_print);
			strcat(TL_TH_to_LCD, "C");
			ST7735_WriteString(0, 108, TL_TH_to_LCD, Font_7x10, ST7735_YELLOW, ST7735_BLUE);

			ST7735_WriteString(0, 120, "CFG REG: ", Font_7x10, ST7735_YELLOW, ST7735_BLUE);
			ST7735_WriteString(56, 120, hex2bin, Font_7x10, ST7735_YELLOW, ST7735_BLUE);

			uint8_t resolution_bits = Cfg_Reg >> 5; 										// Bitwise operation 01000110 >> 5 = 010

			switch (resolution_bits) {
			case 0b00:
				ST7735_WriteString(0, 132, "0b00 = 9 bit res.", Font_7x10, ST7735_YELLOW, ST7735_BLUE);
				break;
			case 0b01:
				ST7735_WriteString(0, 132, "0b01 = 10 bit res.", Font_7x10, ST7735_YELLOW, ST7735_BLUE);
				break;
			case 0b10:
				ST7735_WriteString(0, 132, "0b10 = 11 bit res.", Font_7x10, ST7735_YELLOW, ST7735_BLUE);
				break;
			case 0b11:
				ST7735_WriteString(0, 132, "0b11 = 12 bit res.", Font_7x10, ST7735_YELLOW, ST7735_BLUE);
				break;
			default:
				ST7735_WriteString(35, 0, "ERROR6", Font_7x10, ST7735_YELLOW, ST7735_BLUE);
				break;
			}
		}
	}
}

void DS18B20_Write_Scratchpad() {

	if (DS18B20_Sensor_present == 1) {
		if (ROM_OK_Flag == 1) {
			DS18B20_init();
			ST7735_WriteString(35, 0, "PONG", Font_7x10, ST7735_YELLOW, ST7735_BLUE);

			Write_Byte(0x55);																			// Match ROM command [0x55]
			for (int i = 0; i < 8; i++) {																// Send previously detected
				Write_Byte(ROM_address[i]);																// ROM, all 8 bytes
			}

			Write_Byte(0x4E);    																		//Write Scratchpad command
			Write_Byte(0xFF);    																		//Set TH alarm value 0xFF = +125
			Write_Byte(0x7F);    																		//Set TL alarm value 0x7F = -55C
			Write_Byte(0x7F);    													//Set resolution, 9 bit = 0x1F, 10 bit = 0x3F, 11 bit = 0x5F, 12 bit = 0x7F

		}
	}
}

void DS18B20_Decode_Temperature() {

	if (DS18B20_Sensor_present == 1) {
		if (ROM_OK_Flag == 1) {

			uint16_t Byte0_LSB = (Scratchpad[0]);
			uint16_t Byte1_MSB = (Scratchpad[1]);
			char Byte0_LSB_ITOA[16];
			char Byte1_MSB_ITOA[16];
			itoa(Byte0_LSB, Byte0_LSB_ITOA, 16);
			itoa(Byte1_MSB, Byte1_MSB_ITOA, 16);
			ST7735_WriteString(0, 144, Byte1_MSB_ITOA, Font_7x10, ST7735_YELLOW, ST7735_BLACK);
			ST7735_WriteString(14, 144, ":", Font_7x10, ST7735_YELLOW, ST7735_BLACK);
			ST7735_WriteString(21, 144, Byte0_LSB_ITOA, Font_7x10, ST7735_YELLOW, ST7735_BLACK);

			if ((Scratchpad[1] >> 4) == 0b0000) {

				switch (Cfg_Reg) {
				case 0b01111111:;																	// 12 bit resolution

					uint16_t Positive_temp = ((Scratchpad[1]<<8) + Scratchpad[0]);					// Scratchpad[1]<<8 makes from 0000 000X to X 0000 0000
					float Positive_temp_float = (Positive_temp/16.0);								// Calculating the temperature
					char Positive_temp_FTOC[16];													// then + Scratchpad[0] put bits into it X YYYY YYYY
					sprintf(Positive_temp_FTOC, "T:+%.4fC",Positive_temp_float);						// thus making one complete binary number, which must be /16 for 12 bit resolution
					ST7735_WriteString(49, 144, Positive_temp_FTOC, Font_7x10, ST7735_YELLOW, ST7735_BLUE); // Writing the Positive Temperature Float TO Char value to LCD

					break;

				case 0b01011111:																	// 11 bit resolution

					break;

				case 0b00111111:																	// 10 bit resolution

					break;

				case 0b00011111:																	// 9 bit resolution

					break;

				default:

					ST7735_WriteString(35, 0, "ERROR7", Font_7x10, ST7735_YELLOW, ST7735_BLUE);

					break;
				}

			} else {

				if ((Scratchpad[1] >> 4) == 0b1111) {
					ST7735_WriteString(77, 144, "-", Font_7x10, ST7735_YELLOW, ST7735_BLUE);
				}

			}

		}

	}
}

void Write_Byte(uint8_t bit_to_send_to_DS18B20) {
	Set_DS18B20_Pin_as_Output();

	for (int i = 0; i < 8; i++) {

		if ((bit_to_send_to_DS18B20 & (1 << i)) != 0) 					// writing 1 if the bit is high
				{

			Set_DS18B20_Pin_as_Output();
			HAL_GPIO_WritePin(GPIOA, DS18B20_Data_Pin, 0); 				// writing 1 to the bus
			Delay_in_us(1);  											// wait for 1 us
			Set_DS18B20_Pin_as_Input();  								// set as input
			Delay_in_us(60);  											// wait for 60 us
		}

		else  // writing 0 if the bit is low:
		{

			Set_DS18B20_Pin_as_Output();
			HAL_GPIO_WritePin(GPIOA, DS18B20_Data_Pin, 0); 				// writing 0 to the bus
			Delay_in_us(60);
			Set_DS18B20_Pin_as_Input();
		}
	}
}

uint8_t Read_Byte(void) {
	uint8_t byte = 0;

	Set_DS18B20_Pin_as_Input();

	for (int i = 0; i < 8; i++) {
		Set_DS18B20_Pin_as_Output();   // set as output

		HAL_GPIO_WritePin(GPIOA, DS18B20_Data_Pin, 0);  // pull the data pin LOW
		Delay_in_us(1);  // wait for > 1us

		Set_DS18B20_Pin_as_Input();  // set as input
		if (HAL_GPIO_ReadPin(GPIOA, DS18B20_Data_Pin))  // if the pin is HIGH
				{
			byte |= 1 << i;  // read = 1
		}
		Delay_in_us(50);  // wait for 60 us
	}
	return byte;
}

void Set_DS18B20_Pin_as_Output() {
	GPIO_InitTypeDef GPIO_InitStruct = { 0 }; 				// Setting the DS18B20_Data_Pin as output
	GPIO_InitStruct.Pin = DS18B20_Data_Pin;					// This is already done by the configuration
	GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;				// in the IOC, but just to train how to do it
	GPIO_InitStruct.Pull = GPIO_NOPULL;						// separately repeating here.
	GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
	HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);
}

void Set_DS18B20_Pin_as_Input() {
	GPIO_InitTypeDef GPIO_InitStruct = { 0 }; 				// Setting the DS18B20_Data_Pin as output
	GPIO_InitStruct.Pin = DS18B20_Data_Pin;					// This time setting the pin as input, to see if
	GPIO_InitStruct.Mode = GPIO_MODE_INPUT;					// sensor responds in timely manner.
	GPIO_InitStruct.Pull = GPIO_PULLUP;						// Internal pullup disabled
	GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
	HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);
}

uint8_t CRC_Calculator(uint8_t *String_to_be_CRCed, uint8_t Len) {
	uint8_t i, dat, fb, rom_bit;
	rom_bit = 0;
	crc = 0;
	do {
		dat = String_to_be_CRCed[rom_bit];
		for (i = 0; i < 8; i++) {
			fb = crc ^ dat;
			fb &= 1;
			crc >>= 1;
			dat >>= 1;
			if (fb == 1)
				crc ^= 0x8c;
		}
		rom_bit++;
	} while (rom_bit < Len);
	return crc;
}

//----------------------------------------------------------------------------
/* USER CODE END 4 */

/**
 * @brief  This function is executed in case of error occurrence.
 * @retval None
 */
void Error_Handler(void) {
	/* USER CODE BEGIN Error_Handler_Debug */
	/* User can add his own implementation to report the HAL error return state */
	__disable_irq();
	while (1) {
	}
	/* USER CODE END Error_Handler_Debug */
}

#ifdef  USE_FULL_ASSERT
/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t *file, uint32_t line)
{
  /* USER CODE BEGIN 6 */
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */
  /* USER CODE END 6 */
}
#endif /* USE_FULL_ASSERT */
